/*
 * Copyright (c) 2012, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "curve.h"

#ifdef HAS_CURVE_8_8
uint8_t curve_8_8(const uint8_t *xvals, const uint8_t *yvals,
		uint8_t count, uint8_t in)
{
	// find data points below and above requested x
	uint8_t i = 0;
	while ((i < count) && (xvals[i] < in))
		i++;
	// requested x before curve starts?
	if (i == 0)
		return 0;
	// assign constants for calculation
	const uint8_t x1 = xvals[i - 1];
	const uint8_t y1 = yvals[i - 1];
	const uint8_t x2 = xvals[i];
	const uint8_t y2 = yvals[i];
	// calculate delta values for x and y
	const uint8_t delta_x = x2 - x1;
	const uint8_t delta_y = y2 - y1;
	// calculate y offset from y1 for requested x
	const uint8_t off_y = ((uint16_t)delta_y * (in - x1)) / delta_x;
	// return result: y1 + offset from y1
	return y1 + off_y;
}
#endif

#ifdef HAS_CURVE_16_8
uint8_t curve_16_8(const uint16_t *xvals, const uint8_t *yvals,
		uint8_t count, uint16_t in)
{
	// find data points below and above requested x
	uint8_t i = 0;
	while ((i < count) && (xvals[i] < in))
		i++;
	// requested x before curve starts?
	if (i == 0)
		return 0;
	// assign constants for calculation
	const uint16_t x1 = xvals[i - 1];
	const uint8_t y1 = yvals[i - 1];
	const uint16_t x2 = xvals[i];
	const uint8_t y2 = yvals[i];
	// calculate delta values for x and y
	const uint16_t delta_x = x2 - x1;
	const uint8_t delta_y = y2 - y1;
	// calculate y offset from y1 for requested x
	const uint8_t off_y = ((uint32_t)delta_y * (in - x1)) / delta_x;
	// return result: y1 + offset from y1
	return y1 + off_y;
}
#endif

