/*
 * Copyright (c) 2012, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <avr/eeprom.h>

#include "eeprom.h"

#define EE_ADDR_START  0x40

uint16_t ee_curve_1_x[EE_CURVE_DATA_POINTS];
uint8_t ee_curve_1_y[EE_CURVE_DATA_POINTS];
uint16_t ee_curve_2_x[EE_CURVE_DATA_POINTS];
uint8_t ee_curve_2_y[EE_CURVE_DATA_POINTS];

void ee_load_data(void)
{
	uint8_t *dest = (uint8_t*) EE_ADDR_START;

	eeprom_read_block(ee_curve_1_x, dest, sizeof(ee_curve_1_x));
	dest += sizeof(ee_curve_1_x);

	eeprom_read_block(ee_curve_1_y, dest, sizeof(ee_curve_1_y));
	dest += sizeof(ee_curve_1_y);

	eeprom_read_block(ee_curve_2_x, dest, sizeof(ee_curve_2_x));
	dest += sizeof(ee_curve_2_x);

	eeprom_read_block(ee_curve_2_y, dest, sizeof(ee_curve_2_y));
}

void ee_store_data(void)
{
	uint8_t *dest = (uint8_t*) EE_ADDR_START;

	eeprom_write_block(ee_curve_1_x, dest, sizeof(ee_curve_1_x));
	dest += sizeof(ee_curve_1_x);

	eeprom_write_block(ee_curve_1_y, dest, sizeof(ee_curve_1_y));
	dest += sizeof(ee_curve_1_y);

	eeprom_write_block(ee_curve_2_x, dest, sizeof(ee_curve_2_x));
	dest += sizeof(ee_curve_2_x);

	eeprom_write_block(ee_curve_2_y, dest, sizeof(ee_curve_2_y));
}

