// warning C4100: 'xxx': Unreferenzierter formaler Parameter
#pragma warning(disable: 4100)

#include <windows.h>

#include "resource.h"

#include "conndlg.h"
#include "calibdlg.h"
#include "util.h"
#include "target.h"
#include "maindlg.h"
#include "proto.h"

struct DataBinding {
	int iCtrlId;
	int iChannel;
	int iType;
};

enum {
	TYPE_PWM,
	TYPE_VOLT,
};

static const struct DataBinding BindingTable[] = {
	{ IDC_MAIN_EDT_CH1_PWM,  PARAM_CHAN_A, TYPE_PWM  },
	{ IDC_MAIN_EDT_CH1_VOLT, PARAM_CHAN_A, TYPE_VOLT },
	{ IDC_MAIN_EDT_CH2_PWM,  PARAM_CHAN_B, TYPE_PWM  },
	{ IDC_MAIN_EDT_CH2_VOLT, PARAM_CHAN_B, TYPE_VOLT },
};
#define NUM_DATA_BINDINGS (sizeof(BindingTable)/sizeof(BindingTable[0]))

// Handle to device
static HANDLE hDevice;

// Upate control states
static BOOL UpdateControls(HWND hWnd)
{
	BOOL fConnected = hDevice != INVALID_HANDLE_VALUE;

	// Enabled when device is connected
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_EDT_CH1_PWM), fConnected);
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_EDT_CH1_VOLT), fConnected);
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_BTN_CH1_PWM), fConnected);
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_BTN_CH1_VOLT), fConnected);
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_EDT_CH2_PWM), fConnected);
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_EDT_CH2_VOLT), fConnected);
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_BTN_CH2_PWM), fConnected);
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_BTN_CH2_VOLT), fConnected);
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_BTN_CALIB), fConnected);

	// Enabled when device is not connected
	EnableWindow(GetDlgItem(hWnd, IDC_MAIN_BTN_CONN), !fConnected);

	return TRUE;
}

static BOOL OnBtnConn(HWND hWnd)
{
	struct ConnDlgParam ConnParam;
	HINSTANCE hInst = (HINSTANCE) GetWindowLong(hWnd, GWL_HINSTANCE);
	INT_PTR iRet;

	iRet = DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_CONN_DLG),
		hWnd, ConnDlgProc, (LPARAM) &ConnParam);
	if (iRet == IDOK) {
		hDevice = CommDevOpen(ConnParam.szCommDev);
		UpdateControls(hWnd);
	}

	return TRUE;
}

static BOOL OnBtnCalib(HWND hWnd)
{
	HINSTANCE hInst = (HINSTANCE) GetWindowLong(hWnd, GWL_HINSTANCE);

	DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_CALIB_DLG),
		hWnd, CalibDlgProc, (LPARAM) hDevice);

	return TRUE;
}

static BOOL OnSetData(HWND hWnd, int iChannel, int iType)
{
	BOOL fHandled = FALSE;
	const struct DataBinding *p;
	BOOL fTranslated;
	UINT uValue;

	for (p = BindingTable; p < &BindingTable[NUM_DATA_BINDINGS]; p++) {
		if ((p->iChannel != iChannel) || (p->iType != iType))
			continue;

		uValue = GetDlgItemInt(hWnd, p->iCtrlId, &fTranslated, FALSE);

		if (fTranslated) {
			SetDlgItemText(hWnd, p->iCtrlId, TEXT(""));

			switch (p->iType) {
			case TYPE_PWM:
				TargetSetPwm(hDevice, p->iChannel, uValue);
				fHandled = TRUE;
				break;
			case TYPE_VOLT:
				TargetSetVolt(hDevice, p->iChannel, uValue);
				fHandled = TRUE;
				break;
			}
			
		}
	}

	return fHandled;
}

static BOOL OnInitDialog(HWND hWnd, LPVOID lpParam)
{
	// Device is not valid yet
	hDevice = INVALID_HANDLE_VALUE;
	// Center window on primary desktop
	CenterWindow(hWnd);
	// Initial update of control states
	UpdateControls(hWnd);
	return TRUE;
}

static BOOL OnCommand(HWND hWnd, int iCtrlCode)
{
	BOOL fHandled = FALSE;

	switch (iCtrlCode) {
	case IDC_MAIN_BTN_CONN:
		fHandled = OnBtnConn(hWnd);
		break;
	case IDC_MAIN_BTN_CALIB:
		fHandled = OnBtnCalib(hWnd);
		break;
	case IDC_MAIN_BTN_CH1_PWM:
		fHandled = OnSetData(hWnd, PARAM_CHAN_A, TYPE_PWM);
		break;
	case IDC_MAIN_BTN_CH1_VOLT:
		fHandled = OnSetData(hWnd, PARAM_CHAN_A, TYPE_VOLT);
		break;
	case IDC_MAIN_BTN_CH2_PWM:
		fHandled = OnSetData(hWnd, PARAM_CHAN_B, TYPE_PWM);
		break;
	case IDC_MAIN_BTN_CH2_VOLT:
		fHandled = OnSetData(hWnd, PARAM_CHAN_B, TYPE_VOLT);
		break;
	case IDC_MAIN_BTN_EXIT:
		SendMessage(hWnd, WM_CLOSE, 0, 0);
		fHandled = TRUE;
		break;
	}

	return fHandled;
}

static BOOL OnClose(HWND hWnd)
{
	if (hDevice != INVALID_HANDLE_VALUE) {
		CloseHandle(hDevice);
		hDevice = INVALID_HANDLE_VALUE;
	}
	EndDialog(hWnd, 0);
	return TRUE;
}

INT_PTR CALLBACK MainDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	INT_PTR iResult = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
		iResult = OnInitDialog(hWnd, (LPVOID) lParam);
		break;
	case WM_COMMAND:
		iResult = OnCommand(hWnd, LOWORD(wParam));
		break;
	case WM_CLOSE:
		iResult = OnClose(hWnd);
		break;
	}

	return iResult;
}
