#include "config.h"
#include "target.h"

// Send frame
BOOL TargetSendFrame(HANDLE hDevice, struct frame *f);
// Receive frame
BOOL TargetRecvFrame(HANDLE hDevice, struct frame *f);

void TargetSetPwm(HANDLE hDevice, int iChannel, UINT uPwm)
{
	struct frame f;
	struct param_set_pwm *p = (struct param_set_pwm*) f.param;

	f.type = CMD_SET_PWM;
	p->chan = (uint8_t) iChannel;
	p->pwm = (uint8_t) uPwm;

	TargetSendFrame(hDevice, &f);
	TargetRecvFrame(hDevice, &f);
}

void TargetSetVolt(HANDLE hDevice, int iChannel, UINT uVolt)
{
	struct frame f;
	struct param_set_volt *p = (struct param_set_volt*) f.param;

	f.type = CMD_SET_VOLT;
	p->chan = (uint8_t) iChannel;
	p->volt = (uint16_t) uVolt;

	TargetSendFrame(hDevice, &f);
	TargetRecvFrame(hDevice, &f);
}

BOOL TargetSendFrame(HANDLE hDevice, struct frame *f)
{
	BOOL fFailed = FALSE;

	if (hDevice == INVALID_HANDLE_VALUE)
		fFailed = TRUE;

	if (!fFailed) {
		DWORD dwBytesWritten;

		Sleep(CFG_WRITE_DELAY);
		if (WriteFile(hDevice, &f->type, 1, &dwBytesWritten, NULL) == 0)
			fFailed = TRUE;
		if (dwBytesWritten != 1)
			fFailed = TRUE;
	}

	if (!fFailed) {
		DWORD dwBytesWritten;
		int i;

		for (i = 0; !fFailed && (i < MAX_PARAM_LEN); i++) {
			Sleep(CFG_WRITE_DELAY);
			if (WriteFile(hDevice, &f->param[i], 1, &dwBytesWritten, NULL) == 0)
				fFailed = TRUE;
			if (dwBytesWritten != 1)
				fFailed = TRUE;
		}
	}

	return !fFailed;
}

BOOL TargetRecvFrame(HANDLE hDevice, struct frame *f)
{
	BOOL fFailed = FALSE;

	if (hDevice == INVALID_HANDLE_VALUE)
		fFailed = TRUE;

	if (!fFailed) {
		DWORD dwBytesRead;

		if (ReadFile(hDevice, &f->type, 1, &dwBytesRead, NULL) == 0)
			fFailed = TRUE;
		if (dwBytesRead != 1)
			fFailed = TRUE;
	}

	if (!fFailed) {
		DWORD dwBytesRead;
		int i;

		for (i = 0; !fFailed && (i < MAX_PARAM_LEN); i++) {
			Sleep(100);
			if (ReadFile(hDevice, &f->param[i], 1, &dwBytesRead, NULL) == 0)
				fFailed = TRUE;
			if (dwBytesRead != 1)
				fFailed = TRUE;
		}
	}

	return !fFailed;
}

void TargetGetData(HANDLE hDevice, int iChannel, int iIndex, UINT *lpVolt, UINT *lpPwm)
{
	struct frame f;
	struct param_get_data *p = (struct param_get_data*) f.param;
	struct result_get_data *r = (struct result_get_data*) f.param;

	f.type = CMD_GET_DATA;
	p->chan = (uint8_t) iChannel;
	p->index = (uint8_t) iIndex;
	TargetSendFrame(hDevice, &f);

	TargetRecvFrame(hDevice, &f);
	if (f.type == RES_POSITIVE) {
		*lpVolt = r->xval;
		*lpPwm = r->yval;
	}
}

void TargetSetData(HANDLE hDevice, int iChannel, int iIndex, UINT uVolt, UINT uPwm)
{
	struct frame f;
	struct param_set_data *p = (struct param_set_data*) f.param;
	f.type = CMD_SET_DATA;
	p->chan = (uint8_t) iChannel;
	p->index = (uint8_t) iIndex;
	p->xval = (uint16_t) uVolt;
	p->yval = (uint8_t) uPwm;
	TargetSendFrame(hDevice, &f);

	TargetRecvFrame(hDevice, &f);
}

void TargetLoad(HANDLE hDevice)
{
	struct frame f;
	f.type = CMD_LOAD_DATA;
	TargetSendFrame(hDevice, &f);

	TargetRecvFrame(hDevice, &f);
}

void TargetStore(HANDLE hDevice)
{
	struct frame f;
	f.type = CMD_STORE_DATA;
	TargetSendFrame(hDevice, &f);

	TargetRecvFrame(hDevice, &f);
}
